@@ -24,7 +24,7 @@
 
 #include "dbus-interface.h"
 #include "support.h"
-
+#include <unistd.h>
 /*
 
 To send command to ALL running gnome-mplayers (multihead applications)
@@ -57,34 +57,44 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
     gchar *s = NULL;
     gchar *hrefid = NULL;
     gchar *path = NULL;
+    gchar *keyname = NULL;
     DBusError error;
     DBusMessage *reply_message;
     gchar *path1;
     gchar *path2;
     gchar *path3;
+    gchar *path4;
     GString *xml;
     gchar *xml_string;
-	gint count;
-	GtkTreePath *treepath;
+    gint count;
+    GtkTreePath *treepath;
+    gint source_id;
+	gint bitrate;
 
     message_type = dbus_message_get_type(message);
     sender = dbus_message_get_sender(message);
     destination = dbus_message_get_destination(message);
-/*
-    printf("path=%s; interface=%s; member=%s; data=%s\n",
-               dbus_message_get_path(message),
-               dbus_message_get_interface(message), dbus_message_get_member(message), s);
-*/
+
+    /*
+       printf("path=%s; interface=%s; member=%s; data=%s\n",
+       dbus_message_get_path(message),
+       dbus_message_get_interface(message), dbus_message_get_member(message), s);
+     */
+
     path1 = g_strdup_printf("/control/%i", control_id);
     path2 = g_strdup_printf("/window/%i", embed_window);
     path3 = g_strdup_printf("/pid/%i", getpid());
 
+    path4 = g_strdup_printf("/console/%s", rpconsole);
+
     if (dbus_message_get_path(message)) {
 
         if (g_ascii_strcasecmp(dbus_message_get_path(message), "/") == 0 ||
             g_ascii_strcasecmp(dbus_message_get_path(message), path1) == 0 ||
             g_ascii_strcasecmp(dbus_message_get_path(message), path2) == 0 ||
-            g_ascii_strcasecmp(dbus_message_get_path(message), path3) == 0) {
+            g_ascii_strcasecmp(dbus_message_get_path(message), path3) == 0 ||
+            g_ascii_strcasecmp(dbus_message_get_path(message), path4) == 0 ||
+            g_ascii_strcasecmp(dbus_message_get_path(message), "/org/gnome/SettingsDaemon") == 0) {
 
             // printf("Path matched %s\n", dbus_message_get_path(message));
             if (message_type == DBUS_MESSAGE_TYPE_SIGNAL) {
@@ -93,29 +103,42 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                     dbus_error_init(&error);
                     if (dbus_message_get_args
                         (message, &error, DBUS_TYPE_STRING, &s, DBUS_TYPE_INVALID)) {
-						gtk_list_store_clear(playliststore);
-						selection = NULL;
-						playlist = detect_playlist(s);
-						if (!playlist ) {
-							add_item_to_playlist(s,playlist);
-						} else {
-							if (!parse_playlist(s)) {	
-								add_item_to_playlist(s,playlist);
-							}
-						}
-						if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(playliststore),&iter)) {
-							gtk_tree_model_get(GTK_TREE_MODEL(playliststore), &iter, ITEM_COLUMN,&s, COUNT_COLUMN,&count,PLAYLIST_COLUMN,&playlist,-1);
-							set_media_info(s);
-							play_file(s, playlist);
-							gtk_list_store_set(playliststore,&iter,COUNT_COLUMN,count+1, -1);
-						}
-							
-						if (GTK_IS_TREE_SELECTION(selection)) {
-							treepath = gtk_tree_model_get_path(GTK_TREE_MODEL(playliststore),&iter);
-							gtk_tree_selection_select_path(selection,treepath);	
-							gtk_tree_path_free(treepath);
-						}
-							
+                        if (strlen(s) > 0) {
+                            if (strcmp(rpconsole, "NONE") == 0 || control_instance == FALSE) {
+                                gtk_list_store_clear(playliststore);
+                                gtk_list_store_clear(nonrandomplayliststore);
+                                selection = NULL;
+                                playlist = detect_playlist(s);
+                                if (!playlist) {
+                                    add_item_to_playlist(s, playlist);
+                                } else {
+                                    if (!parse_playlist(s)) {
+                                        add_item_to_playlist(s, playlist);
+                                    }
+                                }
+                                if (gtk_tree_model_get_iter_first
+                                    (GTK_TREE_MODEL(playliststore), &iter)) {
+                                    gtk_tree_model_get(GTK_TREE_MODEL(playliststore), &iter,
+                                                       ITEM_COLUMN, &s, COUNT_COLUMN, &count,
+                                                       PLAYLIST_COLUMN, &playlist, -1);
+                                    set_media_info_name(s);
+                                    play_file(s, playlist);
+                                    gtk_list_store_set(playliststore, &iter, COUNT_COLUMN,
+                                                       count + 1, -1);
+                                }
+
+                                if (GTK_IS_TREE_SELECTION(selection)) {
+                                    treepath =
+                                        gtk_tree_model_get_path(GTK_TREE_MODEL(playliststore),
+                                                                &iter);
+                                    gtk_tree_selection_select_path(selection, treepath);
+									if (GTK_IS_WIDGET(list))
+										gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(list),treepath,NULL,FALSE,0,0);
+                                    gtk_tree_path_free(treepath);
+                                }
+                            }
+                        }
+
                     } else {
                         dbus_error_free(&error);
                     }
@@ -127,24 +150,30 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                     dbus_error_init(&error);
                     if (dbus_message_get_args
                         (message, &error, DBUS_TYPE_STRING, &s, DBUS_TYPE_INVALID)) {
-						gtk_list_store_clear(playliststore);
-						selection = NULL;
-						if (!parse_playlist(s)) {	
-							add_item_to_playlist(s,1);
-						}
-						if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(playliststore),&iter)) {
-							gtk_tree_model_get(GTK_TREE_MODEL(playliststore), &iter, ITEM_COLUMN,&s, COUNT_COLUMN,&count,PLAYLIST_COLUMN,&playlist,-1);
-							set_media_info(s);
-							play_file(s, playlist);
-							gtk_list_store_set(playliststore,&iter,COUNT_COLUMN,count+1, -1);
-						}
+                        gtk_list_store_clear(playliststore);
+                        gtk_list_store_clear(nonrandomplayliststore);
+                        selection = NULL;
+                        if (!parse_playlist(s)) {
+                            add_item_to_playlist(s, 1);
+                        }
+                        if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(playliststore), &iter)) {
+                            gtk_tree_model_get(GTK_TREE_MODEL(playliststore), &iter, ITEM_COLUMN,
+                                               &s, COUNT_COLUMN, &count, PLAYLIST_COLUMN, &playlist,
+                                               -1);
+                            set_media_info_name(s);
+                            play_file(s, playlist);
+                            gtk_list_store_set(playliststore, &iter, COUNT_COLUMN, count + 1, -1);
+                        }
+
+                        if (GTK_IS_TREE_SELECTION(selection)) {
+                            treepath =
+                                gtk_tree_model_get_path(GTK_TREE_MODEL(playliststore), &iter);
+                            gtk_tree_selection_select_path(selection, treepath);
+							if (GTK_IS_WIDGET(list))
+								gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(list),treepath,NULL,FALSE,0,0);
+                            gtk_tree_path_free(treepath);
+                        }
 
-						if (GTK_IS_TREE_SELECTION(selection)) {
-							treepath = gtk_tree_model_get_path(GTK_TREE_MODEL(playliststore),&iter);
-							gtk_tree_selection_select_path(selection,treepath);	
-							gtk_tree_path_free(treepath);
-						}
-							
                     } else {
                         dbus_error_free(&error);
                     }
@@ -156,7 +185,7 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                     dbus_error_init(&error);
                     if (dbus_message_get_args(message, &error, DBUS_TYPE_STRING, &s,
                                               DBUS_TYPE_STRING, &hrefid, DBUS_TYPE_INVALID)) {
-                        make_button(s, hrefid);
+                        make_button(s, g_strdup(hrefid));
                     } else {
                         dbus_error_free(&error);
                     }
@@ -167,13 +196,14 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                     dbus_error_init(&error);
                     if (dbus_message_get_args
                         (message, &error, DBUS_TYPE_STRING, &s, DBUS_TYPE_INVALID)) {
-						add_item_to_playlist(s,0);
-					} else {
+                        if (strlen(s) > 0)
+                            add_item_to_playlist(s, 0);
+                    } else {
                         dbus_error_free(&error);
                     }
                     return DBUS_HANDLER_RESULT_HANDLED;
                 }
-				
+
                 if (g_ascii_strcasecmp(dbus_message_get_member(message), "Close") == 0) {
                     shutdown();
                     return DBUS_HANDLER_RESULT_HANDLED;
@@ -190,30 +220,106 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                     return DBUS_HANDLER_RESULT_HANDLED;
                 }
 
+                if (g_ascii_strcasecmp(dbus_message_get_member(message), "RP_Play") == 0
+                    && idledata != NULL) {
+                    dbus_error_init(&error);
+                    if (dbus_message_get_args
+                        (message, &error, DBUS_TYPE_INT32, &source_id, DBUS_TYPE_INVALID)) {
+                        if (source_id != control_id) {
+                            idledata->fromdbus = TRUE;
+                            g_idle_add(set_play, idledata);
+                        }
+                    } else {
+                        dbus_error_free(&error);
+                    }
+                    return DBUS_HANDLER_RESULT_HANDLED;
+                }
+
                 if (g_ascii_strcasecmp(dbus_message_get_member(message), "Pause") == 0
                     && idledata != NULL) {
                     g_idle_add(set_pause, idledata);
                     return DBUS_HANDLER_RESULT_HANDLED;
                 }
 
+                if (g_ascii_strcasecmp(dbus_message_get_member(message), "RP_Pause") == 0
+                    && idledata != NULL) {
+                    dbus_error_init(&error);
+                    if (dbus_message_get_args
+                        (message, &error, DBUS_TYPE_INT32, &source_id, DBUS_TYPE_INVALID)) {
+                        if (source_id != control_id) {
+                            idledata->fromdbus = TRUE;
+                            g_idle_add(set_pause, idledata);
+                        }
+                    } else {
+                        dbus_error_free(&error);
+                    }
+                    return DBUS_HANDLER_RESULT_HANDLED;
+                }
+
                 if (g_ascii_strcasecmp(dbus_message_get_member(message), "Stop") == 0
                     && idledata != NULL) {
                     g_idle_add(set_stop, idledata);
                     return DBUS_HANDLER_RESULT_HANDLED;
                 }
 
+                if (g_ascii_strcasecmp(dbus_message_get_member(message), "RP_Stop") == 0
+                    && idledata != NULL) {
+                    dbus_error_init(&error);
+                    if (dbus_message_get_args
+                        (message, &error, DBUS_TYPE_INT32, &source_id, DBUS_TYPE_INVALID)) {
+                        if (source_id != control_id) {
+                            idledata->fromdbus = TRUE;
+                            g_idle_add(set_stop, idledata);
+                        }
+                    } else {
+                        dbus_error_free(&error);
+                    }
+                    return DBUS_HANDLER_RESULT_HANDLED;
+                }
+
                 if (g_ascii_strcasecmp(dbus_message_get_member(message), "FastForward") == 0
                     && idledata != NULL) {
                     g_idle_add(set_ff, idledata);
                     return DBUS_HANDLER_RESULT_HANDLED;
                 }
 
+                if (g_ascii_strcasecmp(dbus_message_get_member(message), "RP_FastForward") == 0
+                    && idledata != NULL) {
+                    dbus_error_init(&error);
+                    if (dbus_message_get_args
+                        (message, &error, DBUS_TYPE_INT32, &source_id, DBUS_TYPE_INVALID)) {
+                        if (source_id != control_id) {
+                            idledata->fromdbus = TRUE;
+                            g_idle_add(set_ff, idledata);
+                        }
+                    } else {
+                        dbus_error_free(&error);
+                    }
+                    return DBUS_HANDLER_RESULT_HANDLED;
+                }
+
+
                 if (g_ascii_strcasecmp(dbus_message_get_member(message), "FastReverse") == 0
                     && idledata != NULL) {
                     g_idle_add(set_rew, idledata);
                     return DBUS_HANDLER_RESULT_HANDLED;
                 }
 
+                if (g_ascii_strcasecmp(dbus_message_get_member(message), "RP_FastReverse") == 0
+                    && idledata != NULL) {
+                    dbus_error_init(&error);
+                    if (dbus_message_get_args
+                        (message, &error, DBUS_TYPE_INT32, &source_id, DBUS_TYPE_INVALID)) {
+                        if (source_id != control_id) {
+                            idledata->fromdbus = TRUE;
+                            g_idle_add(set_rew, idledata);
+                        }
+                    } else {
+                        dbus_error_free(&error);
+                    }
+                    return DBUS_HANDLER_RESULT_HANDLED;
+                }
+
                 if (g_ascii_strcasecmp(dbus_message_get_member(message), "Seek") == 0
                     && idledata != NULL) {
                     dbus_error_init(&error);
@@ -228,12 +334,15 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                 }
 
                 if (g_ascii_strcasecmp(dbus_message_get_member(message), "Terminate") == 0) {
-                    shutdown();
-                    dbus_connection_flush(connection);
-                    dbus_connection_remove_filter(connection, filter_func, NULL);
-                    dbus_connection_unref(connection);
-                    connection = NULL;
+/*
+					shutdown();
+					while(gtk_events_pending() || thread != NULL) {
+						gtk_main_iteration();
+					}					
+					dbus_unhook();
                     gtk_main_quit();
+*/
+                    g_idle_add(set_quit, idledata);
                     return DBUS_HANDLER_RESULT_HANDLED;
                 }
 
@@ -250,6 +359,22 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                     return DBUS_HANDLER_RESULT_HANDLED;
                 }
 
+                if (g_ascii_strcasecmp(dbus_message_get_member(message), "RP_Volume") == 0
+                    && idledata != NULL) {
+                    dbus_error_init(&error);
+                    if (dbus_message_get_args
+                        (message, &error, DBUS_TYPE_DOUBLE, &(idledata->volume), DBUS_TYPE_INT32,
+                         &source_id, DBUS_TYPE_INVALID)) {
+                        if (source_id != control_id) {
+                            idledata->fromdbus = TRUE;
+                            g_idle_add(set_volume, idledata);
+                        }
+                    } else {
+                        dbus_error_free(&error);
+                    }
+                    return DBUS_HANDLER_RESULT_HANDLED;
+                }
+
                 if (g_ascii_strcasecmp(dbus_message_get_member(message), "SetFullScreen") == 0
                     && idledata != NULL) {
                     dbus_error_init(&error);
@@ -276,6 +401,22 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                     return DBUS_HANDLER_RESULT_HANDLED;
                 }
 
+                if (g_ascii_strcasecmp(dbus_message_get_member(message), "RP_SetPercent") == 0
+                    && idledata != NULL) {
+                    dbus_error_init(&error);
+                    if (dbus_message_get_args
+                        (message, &error, DBUS_TYPE_DOUBLE, &(idledata->percent), DBUS_TYPE_INT32,
+                         &source_id, DBUS_TYPE_INVALID)) {
+                        if (source_id != control_id) {
+                            idledata->fromdbus = TRUE;
+                            g_idle_add(set_progress_value, idledata);
+                        }
+                    } else {
+                        dbus_error_free(&error);
+                    }
+                    return DBUS_HANDLER_RESULT_HANDLED;
+                }
+
                 if (g_ascii_strcasecmp(dbus_message_get_member(message), "SetCachePercent") == 0
                     && idledata != NULL) {
                     dbus_error_init(&error);
@@ -302,6 +443,54 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                     return DBUS_HANDLER_RESULT_HANDLED;
                 }
 
+                if (g_ascii_strcasecmp(dbus_message_get_member(message), "RP_SetProgressText") == 0
+                    && idledata != NULL) {
+                    dbus_error_init(&error);
+                    if (dbus_message_get_args
+                        (message, &error, DBUS_TYPE_STRING, &s, DBUS_TYPE_INT32, &source_id,
+                         DBUS_TYPE_INVALID)) {
+                        g_strlcpy(idledata->progress_text, s, 1024);
+                        if (source_id != control_id) {
+                            idledata->fromdbus = TRUE;
+                            g_idle_add(set_progress_text, idledata);
+                        }
+                    } else {
+                        dbus_error_free(&error);
+                    }
+                    return DBUS_HANDLER_RESULT_HANDLED;
+                }
+
+
+                if (g_ascii_strcasecmp(dbus_message_get_member(message), "SetMediaLabel") == 0
+                    && idledata != NULL) {
+                    dbus_error_init(&error);
+                    if (dbus_message_get_args
+                        (message, &error, DBUS_TYPE_STRING, &s, DBUS_TYPE_INVALID)) {
+                        g_strlcpy(idledata->media_info, s, 1024);
+                        g_idle_add(set_media_label, idledata);
+                    } else {
+                        dbus_error_free(&error);
+                    }
+                    return DBUS_HANDLER_RESULT_HANDLED;
+                }
+
+                if (g_ascii_strcasecmp(dbus_message_get_member(message), "RP_SetMediaLabel") == 0
+                    && idledata != NULL) {
+                    dbus_error_init(&error);
+                    if (dbus_message_get_args
+                        (message, &error, DBUS_TYPE_STRING, &s, DBUS_TYPE_INT32, &source_id,
+                         DBUS_TYPE_INVALID)) {
+                        g_strlcpy(idledata->media_info, s, 1024);
+                        if (source_id != control_id) {
+                            idledata->fromdbus = TRUE;
+                            g_idle_add(set_media_label, idledata);
+                        }
+                    } else {
+                        dbus_error_free(&error);
+                    }
+                    return DBUS_HANDLER_RESULT_HANDLED;
+                }
+
                 if (g_ascii_strcasecmp(dbus_message_get_member(message), "SetInfo") == 0
                     && idledata != NULL) {
                     dbus_error_init(&error);
@@ -315,6 +504,19 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                     return DBUS_HANDLER_RESULT_HANDLED;
                 }
 
+                if (g_ascii_strcasecmp(dbus_message_get_member(message), "SetURL") == 0) {
+                    dbus_error_init(&error);
+                    if (dbus_message_get_args
+                        (message, &error, DBUS_TYPE_STRING, &s, DBUS_TYPE_INVALID)) {
+                        g_strlcpy(idledata->url, s, 1024);
+                        show_copyurl(idledata);
+                    } else {
+                        dbus_error_free(&error);
+                    }
+                    return DBUS_HANDLER_RESULT_HANDLED;
+                }
+
+
                 if (g_ascii_strcasecmp(dbus_message_get_member(message), "SetShowControls") == 0
                     && idledata != NULL) {
                     dbus_error_init(&error);
@@ -341,6 +543,61 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                     return DBUS_HANDLER_RESULT_HANDLED;
                 }
 
+                if (g_ascii_strcasecmp(dbus_message_get_member(message), "SetGUIState") == 0) {
+                    dbus_error_init(&error);
+                    if (dbus_message_get_args
+                        (message, &error, DBUS_TYPE_INT32, &guistate, DBUS_TYPE_INVALID)) {
+                        g_idle_add(set_gui_state, NULL);
+                    } else {
+                        dbus_error_free(&error);
+                    }
+                    return DBUS_HANDLER_RESULT_HANDLED;
+                }
+
+                if (g_ascii_strcasecmp(dbus_message_get_member(message), "RP_SetGUIState") == 0) {
+                    dbus_error_init(&error);
+                    if (dbus_message_get_args
+                        (message, &error, DBUS_TYPE_INT32, &guistate, DBUS_TYPE_INT32, &source_id,
+                         DBUS_TYPE_INVALID)) {
+                        if (source_id != control_id) {
+                            idledata->fromdbus = TRUE;
+                            g_idle_add(set_gui_state, NULL);
+                        }
+                    } else {
+                        dbus_error_free(&error);
+                    }
+                    return DBUS_HANDLER_RESULT_HANDLED;
+                }
+
+                if (g_ascii_strcasecmp(dbus_message_get_member(message), "MediaPlayerKeyPressed") ==
+                    0) {
+                    dbus_error_init(&error);
+                    if (dbus_message_get_args
+                        (message, &error, DBUS_TYPE_STRING, &s, DBUS_TYPE_STRING, &keyname,
+                         DBUS_TYPE_INVALID)) {
+                        if (g_strncasecmp(keyname, "Play", strlen("Play")) == 0) {
+                            idledata->fromdbus = TRUE;
+                            g_idle_add(set_play, idledata);
+                        }
+                        if (g_strncasecmp(keyname, "Stop", strlen("Stop")) == 0) {
+                            idledata->fromdbus = TRUE;
+                            g_idle_add(set_stop, idledata);
+                        }
+                        if (g_strncasecmp(keyname, "Previous", strlen("Previous")) == 0) {
+                            idledata->fromdbus = TRUE;
+                            g_idle_add(set_prev, idledata);
+                        }
+                        if (g_strncasecmp(keyname, "Next", strlen("Next")) == 0) {
+                            idledata->fromdbus = TRUE;
+                            g_idle_add(set_next, idledata);
+                        }
+                    } else {
+                        dbus_error_free(&error);
+                    }
+                    return DBUS_HANDLER_RESULT_HANDLED;
+                }
+
+
                 if (g_ascii_strcasecmp(dbus_message_get_member(message), "Ping") == 0) {
                     if (control_id != 0) {
                         path = g_strdup_printf("/control/%i", control_id);
@@ -390,6 +647,11 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                                           "    </method>\n"
                                           "    <method name=\"GetCacheSize\">\n"
                                           "    </method>\n"
+                                          "    <method name=\"GetPlayState\">\n"
+                                          "    </method>\n"
+                                          "    <method name=\"GetBitrate\">\n"
+                                          "        <arg name=\"url\" type=\"s\" />\n"
+                                          "    </method>\n"
                                           "    <signal name=\"Open\">\n"
                                           "        <arg name=\"url\" type=\"s\" />\n"
                                           "    </signal>\n"
@@ -482,8 +744,32 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                     dbus_message_unref(reply_message);
                     return DBUS_HANDLER_RESULT_HANDLED;
                 }
+                if (dbus_message_is_method_call(message, "com.gnome.mplayer", "GetPlayState")) {
+                    reply_message = dbus_message_new_method_return(message);
+                    dbus_message_append_args(reply_message, DBUS_TYPE_INT32, &js_state,
+                                             DBUS_TYPE_INVALID);
+                    dbus_connection_send(connection, reply_message, NULL);
+                    dbus_message_unref(reply_message);
+                    return DBUS_HANDLER_RESULT_HANDLED;
+                }
+                if (dbus_message_is_method_call(message, "com.gnome.mplayer", "GetBitrate")) {
+					dbus_error_init(&error);
+					if (dbus_message_get_args(message, &error, DBUS_TYPE_STRING, &s, DBUS_TYPE_INVALID)) {					
+						bitrate = get_bitrate(s);
+					} else {
+						dbus_error_free(&error);
+						bitrate = 0;
+					}
+                    reply_message = dbus_message_new_method_return(message);
+                    dbus_message_append_args(reply_message, DBUS_TYPE_INT32, &bitrate,
+                                             DBUS_TYPE_INVALID);
+                    dbus_connection_send(connection, reply_message, NULL);
+                    dbus_message_unref(reply_message);
+                    return DBUS_HANDLER_RESULT_HANDLED;
+                }
             } else {
-                // printf("path didn't match\n");
+                if (verbose)
+                    printf("path didn't match - %s\n", dbus_message_get_path(message));
             }
 
         }
@@ -501,14 +787,17 @@ void dbus_open_by_hrefid(gchar * hrefid)
     gchar *id;
 
     id = g_strdup(hrefid);
-    printf("requesting id = %s\n", id);
-    path = g_strdup_printf("/control/%i", control_id);
-    message = dbus_message_new_signal(path, "com.gecko.mediaplayer", "RequestById");
-    dbus_message_append_args(message, DBUS_TYPE_STRING, &id, DBUS_TYPE_INVALID);
-    dbus_connection_send(connection, message, NULL);
-    dbus_message_unref(message);
-    g_free(path);
-
+    if (verbose)
+        printf("requesting id = %s\n", id);
+    if (connection != NULL && control_id != 0) {
+        path = g_strdup_printf("/control/%i", control_id);
+        message = dbus_message_new_signal(path, "com.gecko.mediaplayer", "RequestById");
+        dbus_message_append_args(message, DBUS_TYPE_STRING, &id, DBUS_TYPE_INVALID);
+        dbus_connection_send(connection, message, NULL);
+        dbus_message_unref(message);
+        g_free(path);
+    }
+    g_free(id);
 }
 
 void dbus_open_next()
@@ -516,12 +805,13 @@ void dbus_open_next()
     gchar *path;
     DBusMessage *message;
 
-    path = g_strdup_printf("/control/%i", control_id);
-    message = dbus_message_new_signal(path, "com.gecko.mediaplayer", "Next");
-    dbus_connection_send(connection, message, NULL);
-    dbus_message_unref(message);
-    g_free(path);
-
+    if (connection != NULL && control_id != 0) {
+        path = g_strdup_printf("/control/%i", control_id);
+        message = dbus_message_new_signal(path, "com.gecko.mediaplayer", "Next");
+        dbus_connection_send(connection, message, NULL);
+        dbus_message_unref(message);
+        g_free(path);
+    }
 }
 
 void dbus_cancel()
@@ -531,38 +821,154 @@ void dbus_cancel()
     gint id;
 
     id = control_id;
-    path = g_strdup_printf("/control/%i", control_id);
-    message = dbus_message_new_signal(path, "com.gecko.mediaplayer", "Cancel");
-    dbus_message_append_args(message, DBUS_TYPE_INT32, &id, DBUS_TYPE_INVALID);
-    dbus_connection_send(connection, message, NULL);
-    dbus_message_unref(message);
-    g_free(path);
+    if (connection != NULL && control_id != 0) {
+        path = g_strdup_printf("/control/%i", control_id);
+        message = dbus_message_new_signal(path, "com.gecko.mediaplayer", "Cancel");
+        dbus_message_append_args(message, DBUS_TYPE_INT32, &id, DBUS_TYPE_INVALID);
+        dbus_connection_send(connection, message, NULL);
+        dbus_message_unref(message);
+        g_free(path);
+    }
 
 }
 
-void dbus_send_event(gchar *event, gint button)
+void dbus_send_rpsignal(gchar * signal)
+{
+    gchar *path;
+    gchar *localsignal;
+    DBusMessage *message;
+    gint id;
+
+    if (g_strcasecmp(rpconsole, "NONE") == 0)
+        return;
+
+    id = control_id;
+    if (connection != NULL) {
+        path = g_strdup_printf("/console/%s", rpconsole);
+        localsignal = g_strdup(signal);
+        message = dbus_message_new_signal(path, "com.gnome.mplayer", localsignal);
+        dbus_message_append_args(message, DBUS_TYPE_INT32, &id, DBUS_TYPE_INVALID);
+        dbus_connection_send(connection, message, NULL);
+        dbus_message_unref(message);
+        g_free(localsignal);
+        g_free(path);
+    }
+}
+
+void dbus_send_rpsignal_with_int(gchar * signal, int value)
+{
+    gchar *path;
+    gchar *localsignal;
+    DBusMessage *message;
+    gint id;
+
+    if (g_strcasecmp(rpconsole, "NONE") == 0)
+        return;
+
+    id = control_id;
+    if (connection != NULL) {
+        path = g_strdup_printf("/console/%s", rpconsole);
+        localsignal = g_strdup(signal);
+        message = dbus_message_new_signal(path, "com.gnome.mplayer", localsignal);
+        dbus_message_append_args(message, DBUS_TYPE_INT32, &value, DBUS_TYPE_INT32, &id,
+                                 DBUS_TYPE_INVALID);
+        dbus_connection_send(connection, message, NULL);
+        dbus_message_unref(message);
+        g_free(localsignal);
+        g_free(path);
+    }
+}
+
+void dbus_send_rpsignal_with_double(gchar * signal, gdouble value)
+{
+    gchar *path;
+    gchar *localsignal;
+    DBusMessage *message;
+    gint id;
+
+    if (g_strcasecmp(rpconsole, "NONE") == 0)
+        return;
+
+    id = control_id;
+    if (connection != NULL) {
+        path = g_strdup_printf("/console/%s", rpconsole);
+        localsignal = g_strdup(signal);
+        message = dbus_message_new_signal(path, "com.gnome.mplayer", localsignal);
+        dbus_message_append_args(message, DBUS_TYPE_DOUBLE, &value, DBUS_TYPE_INT32, &id,
+                                 DBUS_TYPE_INVALID);
+        dbus_connection_send(connection, message, NULL);
+        dbus_message_unref(message);
+        g_free(localsignal);
+        g_free(path);
+    }
+}
+
+void dbus_send_rpsignal_with_string(gchar * signal, gchar * value)
+{
+    gchar *path;
+    gchar *localsignal;
+    DBusMessage *message;
+    gint id;
+    gchar *localstr;
+
+    if (g_strcasecmp(rpconsole, "NONE") == 0)
+        return;
+
+    id = control_id;
+    if (connection != NULL) {
+        path = g_strdup_printf("/console/%s", rpconsole);
+        localsignal = g_strdup(signal);
+        localstr = g_strdup(value);
+        message = dbus_message_new_signal(path, "com.gnome.mplayer", localsignal);
+        dbus_message_append_args(message, DBUS_TYPE_STRING, &localstr, DBUS_TYPE_INT32, &id,
+                                 DBUS_TYPE_INVALID);
+        dbus_connection_send(connection, message, NULL);
+        dbus_message_unref(message);
+        g_free(localstr);
+        g_free(localsignal);
+        g_free(path);
+    }
+}
+
+void dbus_reload_plugins()
 {
     gchar *path;
-	gchar *localevent;
-	gint localbutton = 0;
     DBusMessage *message;
 
-	localbutton = button;
-	
-	
-	if (connection != NULL) {
-		path = g_strdup_printf("/control/%i", control_id);
-		localevent = g_strdup_printf("%s",event);
-		if (verbose) {
-			//printf("Posting Event %s\n",localevent);
-		}
-		message = dbus_message_new_signal(path, "com.gecko.mediaplayer", "Event");
-		dbus_message_append_args(message, DBUS_TYPE_STRING, &localevent, DBUS_TYPE_INT32, &localbutton, DBUS_TYPE_INVALID);
-		dbus_connection_send(connection, message, NULL);
-		dbus_message_unref(message);
-		g_free(path);
-		g_free(localevent);
-	}
+    if (connection != NULL && control_id != 0) {
+        path = g_strdup_printf("/control/%i", control_id);
+        message = dbus_message_new_signal(path, "com.gecko.mediaplayer", "ReloadPlugins");
+        dbus_connection_send(connection, message, NULL);
+        dbus_message_unref(message);
+        g_free(path);
+    }
+
+}
+
+void dbus_send_event(gchar * event, gint button)
+{
+    gchar *path;
+    gchar *localevent;
+    gint localbutton = 0;
+    DBusMessage *message;
+
+    localbutton = button;
+
+
+    if (connection != NULL && control_id != 0) {
+        path = g_strdup_printf("/control/%i", control_id);
+        localevent = g_strdup_printf("%s", event);
+        if (verbose) {
+            //printf("Posting Event %s\n",localevent);
+        }
+        message = dbus_message_new_signal(path, "com.gecko.mediaplayer", "Event");
+        dbus_message_append_args(message, DBUS_TYPE_STRING, &localevent, DBUS_TYPE_INT32,
+                                 &localbutton, DBUS_TYPE_INVALID);
+        dbus_connection_send(connection, message, NULL);
+        dbus_message_unref(message);
+        g_free(path);
+        g_free(localevent);
+    }
 }
 
 gboolean GetProperty(gchar * property)
@@ -579,13 +985,10 @@ gboolean dbus_hookup(gint windowid, gint controlid)
     gchar *match;
     gchar *path;
     DBusMessage *reply_message;
-    DBusMessage *message;
     gint id;
-    const gchar *app;
-    const gchar *reason;
 
     dbus_error_init(&error);
-    connection = dbus_bus_get(type, &error);
+    connection = dbus_bus_get_private(type, &error);
     if (connection == NULL) {
         printf("Failed to open connection to %s message bus: %s\n",
                (type == DBUS_BUS_SYSTEM) ? "system" : "session", error.message);
@@ -602,13 +1005,22 @@ gboolean dbus_hookup(gint windowid, gint controlid)
 
     match = g_strdup_printf("type='signal',interface='com.gnome.mplayer'");
     dbus_bus_add_match(connection, match, &error);
-    printf("Using match: %s\n", match);
+    if (verbose)
+        printf("Using match: %s\n", match);
+    g_free(match);
+    dbus_error_free(&error);
+
+    match = g_strdup_printf("type='signal',interface='org.gnome.SettingsDaemon'");
+    dbus_bus_add_match(connection, match, &error);
+    if (verbose)
+        printf("Using match: %s\n", match);
     g_free(match);
     dbus_error_free(&error);
 
     dbus_connection_add_filter(connection, filter_func, NULL, NULL);
 
-    printf("Proxy connections and Command connected\n");
+    if (verbose)
+        printf("Proxy connections and Command connected\n");
 
     if (control_id != 0) {
         path = g_strdup_printf("com.gnome.mplayer.cid%i", control_id);
@@ -625,22 +1037,6 @@ gboolean dbus_hookup(gint windowid, gint controlid)
         dbus_bus_request_name(connection, "com.gnome.mplayer", 0, NULL);
     }
 
-    dbus_error_init(&error);
-    message =
-        dbus_message_new_method_call("org.gnome.ScreenSaver", "/org/gnome/ScreenSaver",
-                                     "org.gnome.ScreenSaver", "Inhibit");
-    app = g_strdup_printf("gnome-mplayer");
-    reason = g_strdup_printf("playback");
-    dbus_message_append_args(message, DBUS_TYPE_STRING, &app, DBUS_TYPE_STRING, &reason,
-                             DBUS_TYPE_INVALID);
-    reply_message = dbus_connection_send_with_reply_and_block(connection, message, 200, &error);
-    if (reply_message) {
-        dbus_message_get_args(reply_message, &error, DBUS_TYPE_INT32, &cookie, NULL);
-        dbus_message_unref(reply_message);
-    }
-
-    dbus_message_unref(message);
-    dbus_error_free(&error);
 
     return TRUE;
 }
@@ -649,11 +1045,44 @@ void dbus_unhook()
 {
 
     DBusMessage *message;
-    message =
-        dbus_message_new_method_call("org.gnome.ScreenSaver", "/org/gnome/ScreenSaver",
-                                     "org.gnome.ScreenSaver", "UnInhibit");
-    dbus_message_append_args(message, DBUS_TYPE_INT32, &cookie, DBUS_TYPE_INVALID);
-    dbus_connection_send(connection, message, NULL);
-    dbus_message_unref(message);
 
+    if (connection != NULL) {
+        message =
+            dbus_message_new_method_call("org.gnome.ScreenSaver", "/org/gnome/ScreenSaver",
+                                         "org.gnome.ScreenSaver", "UnInhibit");
+        dbus_message_append_args(message, DBUS_TYPE_INT32, &cookie, DBUS_TYPE_INVALID);
+        dbus_connection_send(connection, message, NULL);
+        dbus_message_unref(message);
+        dbus_connection_close(connection);
+        dbus_connection_unref(connection);
+        connection = NULL;
+    }
+}
+
+void dbus_disable_screensaver()
+{
+    DBusError error;
+    DBusMessage *reply_message;
+    DBusMessage *message;
+    const gchar *app;
+    const gchar *reason;
+
+    if (connection != NULL) {
+        dbus_error_init(&error);
+        message =
+            dbus_message_new_method_call("org.gnome.ScreenSaver", "/org/gnome/ScreenSaver",
+                                         "org.gnome.ScreenSaver", "Inhibit");
+        app = g_strdup_printf("gnome-mplayer");
+        reason = g_strdup_printf("playback");
+        dbus_message_append_args(message, DBUS_TYPE_STRING, &app, DBUS_TYPE_STRING, &reason,
+                                 DBUS_TYPE_INVALID);
+        reply_message = dbus_connection_send_with_reply_and_block(connection, message, 200, &error);
+        if (reply_message) {
+            dbus_message_get_args(reply_message, &error, DBUS_TYPE_INT32, &cookie, NULL);
+            dbus_message_unref(reply_message);
+        }
+
+        dbus_message_unref(message);
+        dbus_error_free(&error);
+    }
 }
