@@ -22,9 +22,12 @@
  * 	Boston, MA  02110-1301, USA.
  */
 
-#include <gnome.h>
+#include <gtk/gtk.h>
+#include <gdk/gdk.h>
+#include <gdk/gdkkeysyms.h>
 #include <glib.h>
 #include <glib/gstdio.h>
+#include <glib/gi18n.h>
 #include <gconf/gconf.h>
 #include <gconf/gconf-client.h>
 #include <gconf/gconf-value.h>
@@ -53,7 +56,9 @@ GtkWidget *menubar;
 GtkMenuItem *menuitem_file;
 GtkMenu *menu_file;
 GtkMenuItem *menuitem_file_open;
+GtkMenuItem *menuitem_file_open_location;
 GtkMenuItem *menuitem_file_open_dvd;
+GtkMenuItem *menuitem_file_open_dvdnav;
 GtkMenuItem *menuitem_file_open_acd;
 GtkMenuItem *menuitem_file_open_playlist;
 GtkMenuItem *menuitem_file_sep1;
@@ -63,22 +68,33 @@ GtkMenuItem *menuitem_file_quit;
 GtkMenuItem *menuitem_edit;
 GtkMenu *menu_edit;
 GtkMenuItem *menuitem_edit_random;
+GtkMenuItem *menuitem_edit_loop;
+GtkMenuItem *menuitem_edit_switch_audio;
 GtkMenuItem *menuitem_edit_sep1;
 GtkMenuItem *menuitem_edit_config;
 GtkMenuItem *menuitem_help;
 GtkMenuItem *menuitem_view;
 GtkMenu *menu_view;
 GtkMenuItem *menuitem_view_playlist;
+GtkMenuItem *menuitem_view_info;
 GtkMenuItem *menuitem_view_sep0;
 GtkMenuItem *menuitem_view_fullscreen;
 GtkMenuItem *menuitem_view_sep1;
 GtkMenuItem *menuitem_view_onetoone;
 GtkMenuItem *menuitem_view_twotoone;
 GtkMenuItem *menuitem_view_onetotwo;
+GtkMenuItem *menuitem_view_sep4;
+GtkMenuItem *menuitem_view_aspect;
+GtkMenu *menu_view_aspect;
+GtkMenuItem *menuitem_view_aspect_default;
+GtkMenuItem *menuitem_view_aspect_four_three;
+GtkMenuItem *menuitem_view_aspect_sixteen_nine;
+GtkMenuItem *menuitem_view_aspect_sixteen_ten;
 GtkMenuItem *menuitem_view_sep2;
-GtkMenuItem *menuitem_view_advanced;
-GtkMenuItem *menuitem_view_sep3;
+GtkMenuItem *menuitem_view_subtitles;
 GtkMenuItem *menuitem_view_controls;
+GtkMenuItem *menuitem_view_sep3;
+GtkMenuItem *menuitem_view_advanced;
 GtkMenu *menu_help;
 GtkMenuItem *menuitem_help_about;
 
@@ -90,6 +106,7 @@ GtkMenuItem *menuitem_play;
 GtkMenuItem *menuitem_pause;
 GtkMenuItem *menuitem_stop;
 GtkMenuItem *menuitem_sep1;
+GtkMenuItem *menuitem_copyurl;
 GtkMenuItem *menuitem_sep2;
 GtkMenuItem *menuitem_sep3;
 GtkMenuItem *menuitem_sep4;
@@ -100,11 +117,14 @@ GtkMenuItem *menuitem_config;
 GtkMenuItem *menuitem_quit;
 gulong delete_signal_id;
 
+GtkWidget *vbox_master;
+GtkWidget *pane;
 GtkWidget *vbox;
 GtkWidget *hbox;
 GtkWidget *controls_box;
 
 GtkWidget *fixed;
+GtkWidget *media_label;
 
 GtkWidget *drawing_area;
 GdkPixbuf *pb_play;
@@ -116,6 +136,7 @@ GdkPixbuf *pb_fs;
 GdkPixbuf *pb_next;
 GdkPixbuf *pb_prev;
 GdkPixbuf *pb_icon;
+GdkPixbuf *pb_logo;
 GdkPixbuf *pb_button;
 GtkWidget *button_event_box;
 GtkWidget *image_button;
@@ -138,6 +159,8 @@ GtkWidget *fs_event_box;
 GtkProgressBar *progress;
 GtkWidget *vol_slider;
 
+gboolean in_button;
+
 GtkWidget *image_play;
 GtkWidget *image_pause;
 GtkWidget *image_stop;
@@ -155,8 +178,26 @@ GtkWidget *config_vo;
 GtkWidget *config_ao;
 GtkWidget *config_cachesize;
 GtkWidget *config_osdlevel;
+
+GtkWidget *config_playlist_visible;
+GtkWidget *config_vertical_layout;
+GtkWidget *config_softvol;
+GtkWidget *config_forcecache;
 GtkWidget *config_verbose;
 
+GtkWidget *config_alang;
+GtkWidget *config_slang;
+
+GtkWidget *config_qt;
+GtkWidget *config_real;
+GtkWidget *config_wmp;
+GtkWidget *config_dvx;
+
+GtkWidget *open_location;
+
+// Playlist container
+GtkWidget *plvbox;
+
 
 GtkAccelGroup *accel_group;
 
@@ -168,6 +209,7 @@ GtkAccelGroup *accel_group;
 #define DRAG_NAME_2		"STRING"
 #define DRAG_INFO_2		2
 
+glong last_movement_time;
 
 gboolean popup_handler(GtkWidget * widget, GdkEvent * event, void *data);
 gboolean delete_callback(GtkWidget * widget, GdkEvent * event, void *data);
@@ -178,9 +220,11 @@ gboolean play_callback(GtkWidget * widget, GdkEventExpose * event, void *data);
 gboolean pause_callback(GtkWidget * widget, GdkEventExpose * event, void *data);
 gboolean stop_callback(GtkWidget * widget, GdkEventExpose * event, void *data);
 gboolean ff_callback(GtkWidget * widget, GdkEventExpose * event, void *data);
+gboolean prev_callback(GtkWidget * widget, GdkEventExpose * event, void *data);
+gboolean next_callback(GtkWidget * widget, GdkEventExpose * event, void *data);
 void vol_slider_callback(GtkRange * range, gpointer user_data);
 gboolean fs_callback(GtkWidget * widget, GdkEventExpose * event, void *data);
-
+gboolean make_panel_and_mouse_visible(gpointer data);
 void menuitem_open_callback(GtkMenuItem * menuitem, void *data);
 void menuitem_quit_callback(GtkMenuItem * menuitem, void *data);
 void menuitem_about_callback(GtkMenuItem * menuitem, void *data);
@@ -190,3 +234,7 @@ void menuitem_stop_callback(GtkMenuItem * menuitem, void *data);
 void menuitem_fs_callback(GtkMenuItem * menuitem, void *data);
 void menuitem_showcontrols_callback(GtkCheckMenuItem * menuitem, void *data);
 void menuitem_quit_callback(GtkMenuItem * menuitem, void *data);
+
+gboolean playlist_drop_callback(GtkWidget * widget, GdkDragContext * dc,
+                                gint x, gint y, GtkSelectionData * selection_data,
+                                guint info, guint t, gpointer data);
