@@ -22,8 +22,16 @@
  * 	Boston, MA  02110-1301, USA.
  */
 
-#include <gnome.h>
+#include <gtk/gtk.h>
 #include <glib.h>
 #include <glib/gstdio.h>
+
+GtkTooltips *playlisttip;
+GtkMenu *playlist_popup_menu;
+GtkMenuItem *playlist_set_subtitle;
+gint window_width, window_height;
+gint filecount;
+
 void update_gui();
 void menuitem_view_playlist_callback(GtkMenuItem * menuitem, void *data);
+void add_item_to_playlist_callback(gpointer data, gpointer user_data);
