@@ -25,6 +25,12 @@
 #include <glib.h>
 #include "support.h"
 #include "common.h"
+#include <unistd.h>
+
+typedef struct _PlayData {
+    gchar filename[4096];
+    gint playlist;
+} PlayData;
 
 gint std_in;
 gint std_out;
